@@ -142,7 +142,9 @@ lrm <- function(y, x, ...) {
 #' @exportS3Method predict lrm
 predict.lrm <- function(object, data = NULL, ...) {
   class(object) <- class(object)[-1]
-  c(cbind(1, as.matrix(data)) %*% object$coefficients)
+  cfx <- object$coefficients
+  cfx[is.na(cfx)] <- 0
+  c(cbind(1, as.matrix(data)) %*% cfx)
 }
 
 #' @exportS3Method residuals lrm
@@ -174,9 +176,12 @@ residuals.glrm <- function(object, response = NULL, data = NULL, ...) {
 }
 
 #' @importFrom glmnet cv.glmnet
+#' @param s Which lambda to use for prediction, defaults to
+#'     \code{"lambda.min"}. See \code{\link[glmnet]{cv.glmnet}}
 #' @rdname regressions
-lasso <- function(y, x, ...) {
+lasso <- function(y, x, s = "lambda.min", ...) {
   obj <- cv.glmnet(y = y, x = as.matrix(x), ...)
+  obj$s <- s
   class(obj) <- c("lasso", class(obj))
   obj
 }
@@ -184,7 +189,7 @@ lasso <- function(y, x, ...) {
 #' @exportS3Method predict lasso
 predict.lasso <- function(object, data = NULL, ...) {
   class(object) <- class(object)[-1]
-  predict(object, newx = as.matrix(data), s = "lambda.1se")[, 1]
+  predict(object, newx = as.matrix(data), s = object$s)[, 1]
 }
 
 #' @exportS3Method residuals lasso
@@ -194,16 +199,17 @@ residuals.lasso <- function(object, response = NULL, data = NULL, ...) {
 }
 
 #' @rdname regressions
-ridge <- function(y, x, ...) {
+ridge <- function(y, x, s = "lambda.min", ...) {
   obj <- cv.glmnet(y = y, x = as.matrix(x), alpha = 0, ...)
+  obj$s <- s
   class(obj) <- c("lasso", class(obj))
   obj
 }
 
 #' @rdname regressions
-postlasso <- function(y, x, ...) {
+postlasso <- function(y, x, s = "lambda.min", ...) {
   obj <- cv.glmnet(y = y, x = as.matrix(x), ...)
-  nz <- which(stats::coef(obj, s = "lambda.1se")[-1] != 0)
+  nz <- which(stats::coef(obj, s = s)[-1] != 0)
   obj <- if (!identical(nz, integer(0))) {
     stats::lm(y ~ x[, nz])
   } else {
@@ -226,6 +232,54 @@ residuals.postlasso <- function(object, response = NULL, data = NULL, ...) {
   .compute_residuals(response, preds)
 }
 
+### unilasso
+unilasso <- function(
+    y, x, loo = TRUE, lower.limits = 0, standardize = FALSE,
+    s = "lambda.min", ...) {
+  ms <- apply(x, 2, \(tx) {
+    stats::lm(y ~ tx)
+  }, simplify = FALSE)
+  xloo <- if (loo) {
+    lapply(ms, \(m) {
+      y - stats::residuals(m) / (1 - stats::hatvalues(m))
+    })
+  } else {
+    lapply(ms, stats::predict)
+  }
+  xloo <- do.call("cbind", xloo)
+  ml <- glmnet::cv.glmnet(
+    x = xloo, y = y, standardize = standardize,
+    lower.limits = lower.limits
+  )
+  cfs <- lapply(ms, stats::coef) |> do.call("rbind", args = _)
+  cf0 <- cfs[, 1]
+  cfx <- cfs[, 2]
+  cfl <- as.double(cfll <- stats::coef(ml, s = s))
+  coef <- c(cfl[1] + sum(cfl[-1] * cf0), cfl[-1] * cfx)
+  names(coef) <- rownames(cfll)
+
+  structure(list(
+    # y = y,
+    # x = x,
+    # xloo = xloo,
+    lm_fits = ms,
+    lasso_fit = ml,
+    cfs = cfs,
+    cfl = cfl,
+    coef = coef
+  ), class = "unilasso")
+}
+
+#' @exportS3Method predict unilasso
+predict.unilasso <- function(object, data, ...) {
+  cbind(1, data) %*% object$coef
+}
+
+#' @exportS3Method residuals unilasso
+residuals.unilasso <- function(object, response, data, ...) {
+  response - stats::predict(object, data, ...)
+}
+
 # Cox ---------------------------------------------------------------------
 
 #' @rdname regressions
@@ -293,7 +347,7 @@ tuned_rf <- function(y, x, max.depths = 1:5,
 #' @rdname regressions
 #' @param nrounds See \code{\link[xgboost]{xgboost}}.
 #' @param verbose See \code{\link[xgboost]{xgboost}}.
-xgb <- function(y, x, nrounds = 2, verbose = 0, ...) {
+xgb <- function(y, x, nrounds = 2L, verbose = 0L, ...) {
   if (requireNamespace("xgboost")) {
     bst <- do.call("xgboost", c(list(
       data = x, label = y, nrounds = nrounds,
@@ -312,19 +366,33 @@ xgb <- function(y, x, nrounds = 2, verbose = 0, ...) {
 #'     \code{\link[xgboost]{xgboost}}.
 #' @param metrics See \code{\link[xgboost]{xgboost}}.
 #' @param nfold Number of folds for \code{nfold}-cross validation.
-tuned_xgb <- function(y, x, etas = c(0.1, 0.5, 1), max_depths = 1:5,
-                      nfold = 5, nrounds = c(2, 10, 50), verbose = 0,
-                      metrics = list("rmse"), ...) {
+#' @param folds Specify folds for cross validation.
+tuned_xgb <- function(y, x, nfold, folds, etas = c(0.1, 0.5, 1),
+                      max_depths = 1:5, nrounds = c(2, 10, 50),
+                      verbose = 0, metrics = list("rmse"), ...) {
   if (requireNamespace("xgboost")) {
+    args <- list(
+      data = x, label = y, verbose = verbose,
+      metrics = metrics
+    )
+    if (missing(folds) && missing(nfold)) {
+      nfold <- 5
+    }
+    if (!missing(nfold)) {
+      args$nfold <- nfold
+    }
+    if (!missing(folds)) {
+      args$folds <- folds
+    }
     cvres <- lapply(etas, \(teta) {
       lapply(max_depths, \(tmd) {
         lapply(nrounds, \(tnr) {
-          cv <- do.call("xgb.cv", c(list(
-            data = x, label = y, nrounds = tnr,
-            verbose = verbose, eta = teta, max_depth = tmd,
-            metrics = metrics, nfold = nfold
-          ), list(...)))
-          err <- mean(cv$evaluation_log[[paste0("test_", metrics[[1]], "_mean")]])
+          args$nrounds <- tnr
+          args$eta <- teta
+          args$max_depth <- tmd
+          cv <- do.call("xgb.cv", c(args, list(...)))
+          ### Take min over boosting iterations
+          err <- min(cv$evaluation_log[[paste0("test_", metrics[[1]], "_mean")]])
           data.frame(
             nrounds = tnr, eta = teta, max_depth = tmd, error = err
           )
@@ -353,3 +421,29 @@ residuals.xgb <- function(object, response = NULL, data = NULL, ...) {
   preds <- predict(object, data = data, ...)
   .compute_residuals(response, preds)
 }
+
+#' @rdname regressions
+lgbm <- function(y, x, nrounds = 100L, verbose = -1L, ...) {
+  if (requireNamespace("lightgbm")) {
+    dtr <- lightgbm::lgb.Dataset(data = x, label = y)
+    bst <- do.call("lgb.train", c(list(
+      data = dtr, nrounds = nrounds,
+      verbose = verbose
+    ), list(...)))
+    class(bst) <- c("lgbm", class(bst))
+    return(bst)
+  }
+  stop("Package `lightgbm` not available.")
+}
+
+#' @exportS3Method predict lgbm
+predict.lgbm <- function(object, data = NULL, ...) {
+  class(object) <- class(object)[-1]
+  predict(object, data, ...)
+}
+
+#' @exportS3Method residuals lgbm
+residuals.lgbm <- function(object, response = NULL, data = NULL, ...) {
+  preds <- predict(object, data = data, ...)
+  .compute_residuals(response, preds)
+}
